package TestTools;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;
import java.util.*;


/**
 * GUIRobot is a front end to the java.awt.Robot class.
 * This program reads a script file (.rsf) that contains
 * commands to the GUI robot. The commands are then
 * translated into method calls to the Robot class.
 * @author David Cumberland.
 * Modifications by John Dalbey, Jason Scaroni
 */
public class SwingRobot
{
    private Hashtable mKeys = new Hashtable();


    public SwingRobot()
    {
        //create hashtable of input characters
        mKeys.put("0", new Integer(KeyEvent.VK_0));
        mKeys.put("1", new Integer(KeyEvent.VK_1));
        mKeys.put("2", new Integer(KeyEvent.VK_2));
        mKeys.put("3", new Integer(KeyEvent.VK_3));
        mKeys.put("4", new Integer(KeyEvent.VK_4));
        mKeys.put("5", new Integer(KeyEvent.VK_5));
        mKeys.put("6", new Integer(KeyEvent.VK_6));
        mKeys.put("7", new Integer(KeyEvent.VK_7));
        mKeys.put("8", new Integer(KeyEvent.VK_8));
        mKeys.put("9", new Integer(KeyEvent.VK_9));
        mKeys.put("a", new Integer(KeyEvent.VK_A));
        mKeys.put("b", new Integer(KeyEvent.VK_B));
        mKeys.put("c", new Integer(KeyEvent.VK_C));
        mKeys.put("d", new Integer(KeyEvent.VK_D));
        mKeys.put("e", new Integer(KeyEvent.VK_E));
        mKeys.put("f", new Integer(KeyEvent.VK_F));
        mKeys.put("g", new Integer(KeyEvent.VK_G));
        mKeys.put("h", new Integer(KeyEvent.VK_H));
        mKeys.put("i", new Integer(KeyEvent.VK_I));
        mKeys.put("j", new Integer(KeyEvent.VK_J));
        mKeys.put("k", new Integer(KeyEvent.VK_K));
        mKeys.put("l", new Integer(KeyEvent.VK_L));
        mKeys.put("m", new Integer(KeyEvent.VK_M));
        mKeys.put("n", new Integer(KeyEvent.VK_N));
        mKeys.put("o", new Integer(KeyEvent.VK_O));
        mKeys.put("p", new Integer(KeyEvent.VK_P));
        mKeys.put("q", new Integer(KeyEvent.VK_Q));
        mKeys.put("r", new Integer(KeyEvent.VK_R));
        mKeys.put("s", new Integer(KeyEvent.VK_S));
        mKeys.put("t", new Integer(KeyEvent.VK_T));
        mKeys.put("u", new Integer(KeyEvent.VK_U));
        mKeys.put("v", new Integer(KeyEvent.VK_V));
        mKeys.put("w", new Integer(KeyEvent.VK_W));
        mKeys.put("x", new Integer(KeyEvent.VK_X));
        mKeys.put("y", new Integer(KeyEvent.VK_Y));
        mKeys.put("z", new Integer(KeyEvent.VK_Z));
        mKeys.put("A", new Integer(KeyEvent.VK_A));
        mKeys.put("B", new Integer(KeyEvent.VK_B));
        mKeys.put("C", new Integer(KeyEvent.VK_C));
        mKeys.put("D", new Integer(KeyEvent.VK_D));
        mKeys.put("E", new Integer(KeyEvent.VK_E));
        mKeys.put("F", new Integer(KeyEvent.VK_F));
        mKeys.put("G", new Integer(KeyEvent.VK_G));
        mKeys.put("H", new Integer(KeyEvent.VK_H));
        mKeys.put("I", new Integer(KeyEvent.VK_I));
        mKeys.put("J", new Integer(KeyEvent.VK_J));
        mKeys.put("K", new Integer(KeyEvent.VK_K));
        mKeys.put("L", new Integer(KeyEvent.VK_L));
        mKeys.put("M", new Integer(KeyEvent.VK_M));
        mKeys.put("N", new Integer(KeyEvent.VK_N));
        mKeys.put("O", new Integer(KeyEvent.VK_O));
        mKeys.put("P", new Integer(KeyEvent.VK_P));
        mKeys.put("Q", new Integer(KeyEvent.VK_Q));
        mKeys.put("R", new Integer(KeyEvent.VK_R));
        mKeys.put("S", new Integer(KeyEvent.VK_S));
        mKeys.put("T", new Integer(KeyEvent.VK_T));
        mKeys.put("U", new Integer(KeyEvent.VK_U));
        mKeys.put("V", new Integer(KeyEvent.VK_V));
        mKeys.put("W", new Integer(KeyEvent.VK_W));
        mKeys.put("X", new Integer(KeyEvent.VK_X));
        mKeys.put("Y", new Integer(KeyEvent.VK_Y));
        mKeys.put("Z", new Integer(KeyEvent.VK_Z));

        //extra characters
        mKeys.put(" ", new Integer(KeyEvent.VK_SPACE));
        mKeys.put("space", new Integer(KeyEvent.VK_SPACE));
        mKeys.put("enter", new Integer(KeyEvent.VK_ENTER));
        mKeys.put("shift", new Integer(KeyEvent.VK_SHIFT));
        mKeys.put("esc", new Integer(KeyEvent.VK_ESCAPE));
        mKeys.put("del", new Integer(KeyEvent.VK_DELETE));
        mKeys.put("backspace", new Integer(KeyEvent.VK_BACK_SPACE));
        mKeys.put("capslock", new Integer(KeyEvent.VK_CAPS_LOCK));
        mKeys.put("ctrl", new Integer(KeyEvent.VK_CONTROL));
        mKeys.put("alt", new Integer(KeyEvent.VK_ALT));
        mKeys.put("tab", new Integer(KeyEvent.VK_TAB));
        mKeys.put(".", new Integer(KeyEvent.VK_PERIOD));
        mKeys.put("down", new Integer(KeyEvent.VK_DOWN));
        mKeys.put("up", new Integer(KeyEvent.VK_UP));
        mKeys.put("left", new Integer(KeyEvent.VK_LEFT));
        mKeys.put("right", new Integer(KeyEvent.VK_RIGHT));
        mKeys.put("-", new Integer(KeyEvent.VK_MINUS));
        mKeys.put("f4", new Integer(KeyEvent.VK_F4));
    }


   public void runScript(String[] args)
   {
        Robot rob = null;          // the instance of AWT Robot
        int delayDuration;         // time to delay after each command
        String line;                // input line from script file
        StringTokenizer strTok;     // to parse input line
        Integer keyEvent;           // user keystroke
        //SwingLOC app;             // the application to be tested

        Class mClass = null;       // This Class represents the user's application

        //process parameters
        if (args.length < 1)
        {
            System.err.println("Usage: provide the application name and a robot script file (.rsf)");
            System.exit(0);
        }
        if (args.length < 2)
        {
            System.err.println("Missing command line parameter: robot script file (.rsf)");
            System.exit(0);
        }


        // Create an instance of the application to be controlled
        try
        {
            // load the class from the file with a new ClassLoader
            String className = args[0];
            // create a new classloader to load the class from the file.
            ClassLoader classLoader = new ClassLoader() {};
            // set mClass to the class loaded from the file.
            mClass = classLoader.loadClass(className);
            // instantiate it
            Object o = mClass.newInstance();
        }
        catch (Exception err)
        {
                        System.out.println("Error: " + err);
                        err.printStackTrace();
        }


        //open the script file for processing
        Reader in = null;
        BufferedReader input = null;
        try
        {
            in = new FileReader (args[1]);
            input = new BufferedReader(in);
        }
        catch (Exception ex)
        {
			ex.printStackTrace();
            System.err.println("Unable to open script file: " + args[1]);
            System.exit(0);
        }


        // create a robot to feed in GUI events
        try
        {
            rob = new Robot();
        }
        catch (AWTException ex)
        {
            System.err.println("Can't start Robot: " + ex);
            System.exit(0);
        }
        rob.delay(2000); //delay to let the application load


        //Process the third command line argument, if it exists
        // which contains a delay duration
        if (args.length >= 3)
        {
            delayDuration = Integer.parseInt(args[2]);
        }
        else
        {
            delayDuration = 500;  // default = .5 seconds
        }
        rob.setAutoDelay(delayDuration);    // set delay that happens after every command

        try
        {
        //process all the lines in file
            while ((line = input.readLine()) != null)
            {
                line = line.trim();    

                //if not a comment and not a short lines (from Unix -> Windows transfer)
                if (line.length() > 0 && 
				 !(line.startsWith("//") || line.startsWith("--")))
                {
                    //process the line, interpreting commands
                    // and calling the Robot methods
                    if (line.startsWith("mouse_move"))
                    {
                        line = line.replace(',', ' ');
                        strTok = new StringTokenizer(line, " ");
                        if (strTok.countTokens() < 3)
                        {
                            throw new Exception("***** invalid script command: "
                             + line);
                        }

                        strTok.nextToken(); //Skip the first token
                        rob.mouseMove(Integer.parseInt(strTok.nextToken()),
                         Integer.parseInt(strTok.nextToken()));
                    }

                    else if (line.startsWith("mouse_press"))
                    {
                        line = line.replace(',', ' ');
                        strTok = new StringTokenizer(line, " ");
                        if (strTok.countTokens() < 3)
                        {
                            throw new Exception("***** invalid script command: "
                             + line);
                        }

                        strTok.nextToken(); //Skip the first token
                        rob.mouseMove(Integer.parseInt(strTok.nextToken()),
                         Integer.parseInt(strTok.nextToken()));
                        rob.mousePress(InputEvent.BUTTON1_MASK);
                        rob.mouseRelease(InputEvent.BUTTON1_MASK);
                    }

                    else if (line.startsWith("mouse_down"))
                    {
                        rob.mousePress(InputEvent.BUTTON1_MASK);
                    }

                    else if (line.startsWith("mouse_up"))
                    {
                        rob.mouseRelease(InputEvent.BUTTON1_MASK);
                    }

                    else if (line.startsWith("press_key"))
                    {
                        strTok = new StringTokenizer(line, " ");
                        if (strTok.countTokens() < 2)
                        {
                            throw new Exception("***** invalid script command: "
                             + line);
                        }

                        strTok.nextToken(); //Skip the first token
                        keyEvent = (Integer) mKeys.get(strTok.nextToken().toLowerCase());
                        if (keyEvent == null)
                        {
                            throw new Exception("***** unrecognized key name: "
                             + line);
                        }

                        rob.keyPress(keyEvent.intValue());
                    }

                    else if (line.startsWith("release_key"))
                    {
                        strTok = new StringTokenizer(line, " ");
                        if (strTok.countTokens() < 2)
                        {
                            throw new Exception("***** invalid script command: "
                             + line);
                        }

                        strTok.nextToken(); //Skip the first token
                        keyEvent = (Integer) mKeys.get(strTok.nextToken().toLowerCase());
                        if (keyEvent == null)
                        {
                            throw new Exception("***** unrecognized key name: "
                             + line);
                        }
                        rob.keyRelease(keyEvent.intValue());
                    }

                    else if (line.startsWith("type_key"))
                    {
                        strTok = new StringTokenizer(line, " ");
                        if (strTok.countTokens() < 2)
                        {
                            throw new Exception("***** invalid script command: "
                             + line);
                        }

                        strTok.nextToken(); //Skip the first token
                        keyEvent = (Integer) mKeys.get(strTok.nextToken().toLowerCase());
                        if (keyEvent == null)
                        {
                            throw new Exception("***** unrecognized key name: "
                             + line);
                        }
                        rob.keyPress(keyEvent.intValue());
                        rob.keyRelease(keyEvent.intValue());
                    }

                    else if (line.startsWith("type_string"))
                    {
                        line = line.substring(12).trim();
                        rob.setAutoDelay(1);    // turn off delay for entering string

                        for (int index = 0; index < line.length(); index++)
                        {
                            char letter = line.charAt(index);
                            if (letter >= 'A' && letter <= 'Z')
                            {
                                rob.keyPress(KeyEvent.VK_SHIFT);
                            }
                            keyEvent = (Integer) mKeys.get(String.valueOf(letter));
                            rob.keyPress(keyEvent.intValue());
                            rob.keyRelease(keyEvent.intValue());
                            rob.keyRelease(KeyEvent.VK_SHIFT);
                        }

                        rob.setAutoDelay(delayDuration);    // restore delay
                    }

                    else if (line.startsWith("wait"))
                    {
                        strTok = new StringTokenizer(line, " ");
                        if (strTok.countTokens() < 2)
                        {
                            throw new Exception("***** invalid script command: "
                             + line);
                        }

                        strTok.nextToken(); //Skip the first token
                        rob.delay(Integer.parseInt(strTok.nextToken()));
                    }

                    else
                    {
                        throw new Exception("***** invalid script command: " + line);
                    }

                    //rob.delay(500);
                }
            }
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
        }

        //close the file
        try
        {
            if (input != null)
            {
                input.close();
            }
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
        }

        //make sure all special mKeys have been turned off
        rob.keyRelease(KeyEvent.VK_CONTROL);
        rob.keyRelease(KeyEvent.VK_ALT);
    }


    public static void main(String[] args) throws AWTException
    {
        new SwingRobot().runScript(args);
        System.exit(0);
    }
}
